/*Copyright (C) 2010-2023 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_KICK_REASON
#define DEF_KICK_REASON

#include <string>
#include <cstdint>

/// Encodes the reasons describing why a peer was disconnected
enum class KickReason : std::uint32_t
{
    ConnectionLost = 0,
    VersionMismatch = 1,
    ServerIsFull = 2,
    ServerNoCharacter = 3,
    Inactivity = 4,
    IllegalCommand = 5,
    UnintelligibleByServer = 6,
    UnintelligibleByClient = 7,
    ConnectionTimeout = 8,
    ClientNoArena = 9,
    ClientNoCharacter = 10,
    ClientDecision = 11,
    EnetError = 12,
    Unknown = 13
};

namespace KickInfo
{
    inline KickReason getReason(std::uint32_t reasonCode)
    {
        if (reasonCode < static_cast<std::uint32_t>(KickReason::Unknown))
            return static_cast<KickReason>(reasonCode);

        return KickReason::Unknown;
    }

    inline std::string describe(KickReason reason)
    {
        if (reason == KickReason::ConnectionLost)
            return "Connection lost";
        else if (reason == KickReason::VersionMismatch)
            return "Incompatible client and server versions";
        else if (reason == KickReason::ServerIsFull)
            return "The server is full";
        else if (reason == KickReason::ServerNoCharacter)
            return "Requested character unsupported by server";
        else if (reason == KickReason::Inactivity)
            return "Disconnected for inactivity";
        else if (reason == KickReason::IllegalCommand)
            return "Illegal command";
        else if (reason == KickReason::UnintelligibleByServer)
            return "The server failed to interpret a packet";
        else if (reason == KickReason::UnintelligibleByClient)
            return "Could not interpret a server packet";
        else if (reason == KickReason::ConnectionTimeout)
            return "Connection timeout";
        else if (reason == KickReason::ClientNoArena)
            return "Unsupported arena sent by server";
        else if (reason == KickReason::ClientNoCharacter)
            return "Unsupported character sent by server";
        else if (reason == KickReason::ClientDecision)
            return "Decided by client";
        else if (reason == KickReason::EnetError)
            return "ENet error while connecting";

        return "Unknown reason";
    }
}

#endif
